package hondana.businesslogic;

import java.util.Map;

import hondana.exception.BusinessException;
import hondana.view.RegistBookConfirmView;
import hondana.view.RegistBookInputView;

/**
 * 登録する本の入力
 */
public class RegistBookInput extends AbstractBusinessLogic {

    {
        // クエリーランナーを取得。
        requiresQueryRunner = true;
    }

    public Object execute() {
        RegistBookInputView in = (RegistBookInputView) view;

        // ISBN、タイトル、著者、ジャンルを取得。
        String isbn = in.getIsbn();
        String title = in.getTitle();
        String author = in.getAuthor();
        String category = in.getCategory();

        // 同じ本が既に登録されていないかチェック。
        Integer count = (Integer) qr.getOne(
                "select count(ISBN) from BOOKS where ISBN = ?", isbn);

        if (count.intValue() > 0)
            throw BusinessException.RB001;

        // カテゴリ名を取得。
        Map categories = session.getCategories();
        String categoryName = (String) categories.get(category);

        // ビューを生成して、プロパティをセット。
        RegistBookConfirmView out = new RegistBookConfirmView();
        out.setIsbn(isbn);
        out.setTitle(title);
        out.setAuthor(author);
        out.setCategory(category);
        out.setCategoryName(categoryName);

        // ビューを返す。
        return out;
    }

    public Object error(Exception ex) {
        // ビューに「戻る」フラグをセット。
        view.setReturned(true);
        // ビューを返す。
        return view;
    }

}