package hondana.businesslogic;

import hondana.exception.BusinessException;
import hondana.object.User;
import hondana.view.RegistBookComplateView;
import hondana.view.RegistBookConfirmView;
import hondana.view.View;

/**
 * 本の登録
 */
public class RegistBookRegist extends AbstractBusinessLogic {

    {
        // クエリーランナーを取得。
        requiresQueryRunner = true;
    }

    public Object execute() {
        RegistBookConfirmView in = (RegistBookConfirmView) view;

        // ISBN、タイトル、著者、ジャンルを取得。
        String isbn = in.getIsbn();
        String title = in.getTitle();
        String author = in.getAuthor();
        String category = in.getCategory();

        // 同じ本が既に登録されていないかチェック。
        Integer count = (Integer) qr.getOne(
                "select count(ISBN) from BOOKS where ISBN = ?", isbn);

        if (count.intValue() > 0)
            throw BusinessException.RB001;

        // 本を登録。
        Object[] bparams = { isbn, title, author, category };
        qr.update("insert into BOOKS VALUES (?, ?, ?, ?, SYSDATE, SYSDATE)",
                bparams);

        // 持ち主を登録。
        User user = session.getUser();
        Object[] oparams = { isbn, user.getUserId() };
        qr.update("insert into OWNERSHIP VALUES (?, ?, SYSDATE, SYSDATE)",
                oparams);

        // セッションからジャンルリストを削除。
        session.removeCategories();

        // ビューを生成して、タイトルをセット。
        RegistBookComplateView out = new RegistBookComplateView();
        out.setTitle(title);

        // ビューを返す。
        return out;
    }

    public Object error(Exception ex) {
        // スタックからビューを取得。
        View out = stack.pop();
        // ビューに「戻る」フラグをセット。
        out.setReturned(true);
        // ビューを返す。
        return out;
    }

}