package hondana.util.database.impl;

import hondana.exception.SystemException;

import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.apache.commons.dbutils.BasicRowProcessor;

/**
 * 行プロセッサ
 */
class AppRowProcessor extends BasicRowProcessor {

    public Map toMap(ResultSet rs) throws SQLException {
        LockedCaseInsensitiveHashMap result = new LockedCaseInsensitiveHashMap();
        ResultSetMetaData rsmd = rs.getMetaData();
        int cols = rsmd.getColumnCount();

        for (int i = 1; i <= cols; i++) {
            Object value = rs.getObject(i);
            if (value instanceof Date)
                value = new hondana.util.Date((Date) value);
            result.put(rsmd.getColumnName(i), value);
        }

        result.lock();
        return result;
    }

    /////////////////////////////////////////////////////////////////
    // locked map

    private static class LockedCaseInsensitiveHashMap extends HashMap {

        private boolean locked = false;

        private void lock() {
            locked = true;
        }

        public boolean containsKey(Object key) {
            return super.containsKey(key.toString().toLowerCase());
        }

        public Object get(Object key) {
            if (locked && !containsKey(key))
                throw new SystemException(
                        "accessed the key which does not exist '" + key + "'");

            return super.get(key.toString().toLowerCase());
        }

        public Object put(Object key, Object value) {
            if (locked)
                throw new SystemException("cannot modify locked map '"
                        + this.getClass().getName() + "'");

            return super.put(key.toString().toLowerCase(), value);
        }

        public void putAll(Map m) {
            if (locked)
                throw new SystemException("cannot modify locked map '"
                        + this.getClass().getName() + "'");

            Iterator iter = m.keySet().iterator();
            while (iter.hasNext()) {
                Object key = iter.next();
                Object value = m.get(key);
                this.put(key, value);
            }
        }

        public Object remove(Object key) {
            if (locked)
                throw new SystemException("cannot modify locked map '"
                        + this.getClass().getName() + "'");

            return super.remove(key.toString().toLowerCase());
        }
    }

}