package hondana.util.database.impl;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

import org.apache.commons.dbutils.QueryRunner;
import org.apache.commons.dbutils.handlers.MapListHandler;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import hondana.exception.SQLRuntimeException;

/**
 * クエリーランナー(実装)
 */
public class QueryRunnerImpl implements hondana.util.database.QueryRunner {

    private static Log logger = LogFactory.getLog(QueryRunnerImpl.class);

    private static Class driver = null;

    private Connection conn = null;

    private QueryRunner qr = new QueryRunner();

    /////////////////////////////////////////////////////////////////
    // static method

    public static void registerDriver(String name) throws SQLRuntimeException {
        try {
            if (driver == null) {
                driver = Class.forName(name);
                logger.debug("regist driver: " + name);
            }
        } catch (ClassNotFoundException e) {
            throw new SQLRuntimeException(e);
        }
    }

    /////////////////////////////////////////////////////////////////
    // connection operator

    public QueryRunnerImpl(String url, String user, String password)
            throws SQLRuntimeException {
        try {
            logger.debug("create QueryRunner 'url=" + url + ", user=" + user
                    + ", password=" + password + ", driver=" + driver + "'");
            conn = DriverManager.getConnection(url, user, password);
            conn.setAutoCommit(false);
        } catch (SQLException e) {
            throw new SQLRuntimeException(e);
        }
    }

    public void close() throws SQLRuntimeException {
        try {
            logger.debug("connection cloase");
            conn.close();
        } catch (SQLException e) {
            throw new SQLRuntimeException(e);
        }
    }

    public void commit() throws SQLRuntimeException {
        try {
            logger.debug("transaction commit");
            conn.commit();
        } catch (SQLException e) {
            throw new SQLRuntimeException(e);
        }
    }

    public void rollback() throws SQLRuntimeException {
        try {
            logger.debug("transaction rollback");
            conn.rollback();
        } catch (SQLException e) {
            throw new SQLRuntimeException(e);
        }
    }

    /////////////////////////////////////////////////////////////////
    // query

    public Map[] query(String sql) throws SQLRuntimeException {
        try {
            logger.debug("sql issue: " + sql);
            List rows = (List) qr.query(conn, sql, new MapListHandler(
                    new AppRowProcessor()));
            return (Map[]) rows.toArray(new Map[rows.size()]);
        } catch (SQLException e) {
            throw new SQLRuntimeException(e);
        }
    }

    public Map[] query(String sql, Object param) throws SQLRuntimeException {
        try {
            logger.debug("sql issue: " + sql + " [" + param + "]");
            List rows = (List) qr.query(conn, sql, param, new MapListHandler(
                    new AppRowProcessor()));
            return (Map[]) rows.toArray(new Map[rows.size()]);
        } catch (SQLException e) {
            throw new SQLRuntimeException(e);
        }
    }

    public Map[] query(String sql, Object[] params) throws SQLRuntimeException {
        try {
            logger.debug("sql issue: " + sql + " " + Arrays.asList(params));
            List rows = (List) qr.query(conn, sql, params, new MapListHandler(
                    new AppRowProcessor()));
            return (Map[]) rows.toArray(new Map[rows.size()]);
        } catch (SQLException e) {
            throw new SQLRuntimeException(e);
        }
    }

    /////////////////////////////////////////////////////////////////
    // get row

    public Map getRow(String sql) throws SQLRuntimeException {
        return query(sql)[0];
    }

    public Map getRow(String sql, Object param) throws SQLRuntimeException {
        return query(sql, param)[0];
    }

    public Map getRow(String sql, Object[] params) throws SQLRuntimeException {
        return query(sql, params)[0];
    }

    /////////////////////////////////////////////////////////////////
    // get one

    public Object getOne(String sql) throws SQLRuntimeException {
        Map row = getRow(sql);
        return row.values().iterator().next();
    }

    public Object getOne(String sql, Object param) throws SQLRuntimeException {
        Map row = getRow(sql, param);
        return row.values().iterator().next();
    }

    public Object getOne(String sql, Object[] params)
            throws SQLRuntimeException {
        Map row = getRow(sql, params);
        return row.values().iterator().next();
    }

    /////////////////////////////////////////////////////////////////
    // update

    public int update(String sql) throws SQLRuntimeException {
        try {
            logger.debug("sql issue: " + sql);
            return qr.update(conn, sql);
        } catch (SQLException e) {
            throw new SQLRuntimeException(e);
        }
    }

    public int update(String sql, Object param) throws SQLRuntimeException {
        try {
            logger.debug("sql issue: " + sql + " [" + param + "]");
            return qr.update(conn, sql, param);
        } catch (SQLException e) {
            throw new SQLRuntimeException(e);
        }
    }

    public int update(String sql, Object[] params) throws SQLRuntimeException {
        try {
            logger.debug("sql issue: " + sql + " " + Arrays.asList(params));
            return qr.update(conn, sql, params);
        } catch (SQLException e) {
            throw new SQLRuntimeException(e);
        }
    }

}